#include "IGame.h"

GameSettings darts_settings[] = {
	{ "game", "game", "301|501|701|901", 0 },
	{ "dblin", "double in", 0, 0 },
	{ "dblout", "double out", 0, 0 },
	{ 0, 0, 0, 0 }
};


#define BMP_BOARD                        0        /* BMP  */
#define BMP_DART                         1        /* BMP  */
#define BMP_HOLDER                       2        /* BMP  */
#define BMP_MARK                         3        /* BMP  */
#define BMP_NUMBERS                      4        /* BMP  */
#define BMP_PAD                          5        /* BMP  */
#define BMP_SCORE1                       6        /* BMP  */
#define BMP_SCORE2                       7        /* BMP  */
#define SMP_25                           8        /* SAMP */
#define SMP_50                           9        /* SAMP */
#define SMP_BUST                         10       /* SAMP */
#define SMP_DOUBLE                       11       /* SAMP */
#define SMP_HIT                          12       /* SAMP */
#define SMP_TRIPLE                       13       /* SAMP */
#define SMP_WIN                          14       /* SAMP */

DATAFILE *darts_data = NULL;

class Params {
	public:
		static int w;
		static int h;
		static int bpp;
		static int fullscreen;
		static int fps;
		static int rim[6];
		static int difficulty;
		static int maxPoints;
		static int doubleIn;
		static int doubleOut;

	public:
		static void Load(const char *file);
		static void Save(const char *file);
};

int Params::w = 480;
int Params::h = 320;
int Params::bpp = 32;
int Params::fullscreen = 0;
int Params::fps = 50;
int Params::rim[] = {3,8,44,49,74,79};
int Params::difficulty = 2;
int Params::maxPoints = 501;
int Params::doubleIn = 0;
int Params::doubleOut = 1;


void Params::Load(const char *file) {
	set_config_file(file);

	w = get_config_int("GENERAL", "w", w);
	h = get_config_int("GENERAL", "h", h);
	bpp = get_config_int("GENERAL", "bpp", bpp);
	fullscreen = get_config_int("GENERAL", "fullscreen", fullscreen);
	fps = get_config_int("GENERAL", "fps", fps);
	difficulty = get_config_int("GAME", "difficulty", difficulty);
	maxPoints = get_config_int("GAME", "maxPoints", maxPoints);
	doubleIn = get_config_int("GAME", "doubleIn", doubleIn);
	doubleOut = get_config_int("GAME", "doubleOut", doubleOut);

	char *name[] = { "rim1", "rim2", "rim3", "rim4", "rim5", "rim6" };
	for (int i=0; i<6; ++i) {
		rim[i] = get_config_int("BOARD", name[i], rim[i]);
	}
}

void Params::Save(const char *file) {
	set_config_file(file);

	set_config_int("GENERAL", "w", w);
	set_config_int("GENERAL", "h", h);
	set_config_int("GENERAL", "bpp", bpp);
	set_config_int("GENERAL", "fps", fps);
	set_config_int("GENERAL", "fullscreen", fullscreen);
	set_config_int("GAME", "difficulty", difficulty);
	set_config_int("GAME", "maxPoints", maxPoints);
	set_config_int("GAME", "doubleIn", doubleIn);
	set_config_int("GAME", "doubleOut", doubleOut);

	char *name[] = { "rim1", "rim2", "rim3", "rim4", "rim5", "rim6" };
	for (int i=0; i<6; ++i) {
		set_config_int("BOARD", name[i], rim[i]);
	}

	flush_config_file();
}


class Sound {
	protected:
		static DATAFILE *data;

	public:
		static void Initialize();
		static void Destroy();
		static void Play(int i, int freq = 1000);
};


DATAFILE *Sound::data = NULL;

void Sound::Initialize() {
	data = darts_data;
	//data = load_datafile("sounds.dat");
}


void Sound::Destroy() {
	/*
	if (data) {
		unload_datafile(data);
	}
	*/
}


void Sound::Play(int i, int freq) {
	if (data) {
		if (i == SMP_WIN || i == SMP_BUST) {
			stop_sample((SAMPLE *)data[SMP_25].dat);
			stop_sample((SAMPLE *)data[SMP_50].dat);
			stop_sample((SAMPLE *)data[SMP_DOUBLE].dat);
			stop_sample((SAMPLE *)data[SMP_TRIPLE].dat);
		}
		play_sample((SAMPLE *)data[i].dat, 255, 128, freq, 0);
	}
}


class Object {
	protected:
		float x, y;

	public:
		Object(float x_, float y_) : x(x_), y(y_) {}
		virtual ~Object() {}

		virtual void Draw(BITMAP *) {}
		virtual void Update() {}
};


class Player {
	protected:
		int darts_;
		int score_;
		int saved_;

	public:
		Player();
		~Player();

		void Throw();
		int DartsLeft();
		int Score();
		void AwardPoints(int pts, int factor);
		void StartTurn();
		void Reset();
};


Player::Player() : darts_(0), score_(Params::maxPoints), saved_(-1) {}
Player::~Player() {}


void Player::Reset() {
	darts_ = 0;
	score_ = Params::maxPoints;
	saved_ = -1;
}


void Player::Throw() {
	--darts_;
}


int Player::DartsLeft() {
	return darts_;
}


int Player::Score() {
	return score_;
}


void Player::AwardPoints(int pts, int f) {
	if (score_ - pts < 0) {
		darts_ = 0;
		Sound::Play(SMP_BUST);
		if (saved_ != -1) {
			score_ = saved_;
			saved_ = -1;
		}
	}
	else {
		if (saved_ == -1) {
			saved_ = score_;
		}
		if (score_ == Params::maxPoints) {
			if ((Params::doubleIn && f == 2) || !Params::doubleIn) {
				score_ -= pts;
			}
		}
		else {
			score_ -= pts;
		}
		if (Params::doubleOut) {
			if ((score_ == 0 && f != 2) || score_ == 1) {
				Sound::Play(SMP_BUST);
				score_ = saved_;
				darts_ = 0;
			}
		}
		if (!DartsLeft()) {
			saved_ = -1;
		}
	}
}


void Player::StartTurn() {
	darts_ = 3;
}


class Pad : public Object {
	protected:
		BITMAP *back;
		int size;

	public:
		Pad(int x, int y, int size);
		~Pad();

		void Draw(BITMAP *buffer);
		bool GetMousePos(int &x, int &y, int mouse_x, int mouse_y, int mouse_b);
		bool CanThrow(int mouse_x, int mouse_y);
};



Pad::Pad(int x, int y, int size_) : Object(x, y), size(size_) {
	back = (BITMAP *)darts_data[BMP_PAD].dat;
	/*
	back = load_bitmap("pad.tga", NULL);
	if (!back) {
		back = create_bitmap(160, 160);
		clear_to_color(back, makecol(128,128,128));
	}
	*/
}


Pad::~Pad() {
	//destroy_bitmap(back);
}


void Pad::Draw(BITMAP *buffer) {
	stretch_blit(back, buffer, 0, 0, back->w, back->h, (int)x, (int)y, size, size);
}


bool Pad::GetMousePos(int &x2, int &y2, int mouse_x, int mouse_y, int mouse_b) {
	int mx = mouse_x - (int)x;
	int my = mouse_y - (int)y;
	if (mx > size || my > size) {
		x2 = 0;
		y2 = 0;
		return false;
	}

	x2 = mx*Params::h/size;
	y2 = my*Params::h/size;

	return mouse_b ? false : true;
}


bool Pad::CanThrow(int mouse_x, int mouse_y) {
	return (mouse_x >= (int)x && mouse_y >= (int)y + size);
}



class Board : public Object {
	protected:
		BITMAP *board;
		BITMAP *bmp;
		BITMAP *mark;
		int rim[6];

		void GetPoint(float a, float r, int p, float &x, float &y);
		void MarkHit(int x, int y);

	public:
		Board(int size);
		~Board();

		void Draw(BITMAP *buffer);
		int GetPoints(int mx, int my, int &factor);
		void DrawBoard();
};


#include <math.h>

Board::Board(int size) : Object(0.0f, 0.0f) {
	board = create_bitmap(size, size);
	bmp = (BITMAP *)darts_data[BMP_BOARD].dat;
	mark = (BITMAP *)darts_data[BMP_MARK].dat;

	/*
	bmp = load_bitmap("board.tga", NULL);
	mark = load_bitmap("mark.tga", NULL);
	if (!mark) {
		mark = create_bitmap(11,11);
		clear_to_color(mark, makecol(255,0,255));
		circlefill(mark, 5,5,5, makecol(0,255,0));
	}
	*/

	for (int i=0; i<6; ++i) {
		rim[i] = Params::rim[i];
	}
	DrawBoard();
}


Board::~Board() {
	destroy_bitmap(board);

	/*
	if (bmp) destroy_bitmap(bmp);
	destroy_bitmap(mark);
	*/
}


void Board::Draw(BITMAP *buffer) {
	blit(board, buffer, 0, 0, (int)x, (int)y, board->w, board->h);
}


inline float FRAC(float x, int p) { return p*x/100.0f; }
inline float RAD(float x) { return x*3.14159f/180.0f; }
inline float DEG(float x) { return x*180.0f/3.14159f; }

void Board::GetPoint(float a, float r, int p, float &x2, float &y2) {
	x2 = (r + FRAC(r,p) * cos(RAD(a)));
	y2 = (r + FRAC(r,p) * sin(RAD(a)));
}

void Board::DrawBoard() {
	if (bmp) {
		stretch_blit(bmp, board, 0, 0, bmp->w, bmp->h, 0, 0, board->w, board->h);
		return;
	}

	//clear_to_color(board, makecol(0,16,32));
	//int w = board->w-1;
	//int h = board->h-1;
	int r = (board->w)>>1;

	int black = makecol(0,0,0);
	int red = makecol(255,0,0);
	int yellow = makecol(255,255,0);
	int white = makecol(255,255,255);

	circlefill(board, r, r, r, black);

	float da = 18;
	float alpha = 9;
	float x2, y2;
	for (int i=0; i<20; ++i) {
		GetPoint(alpha, r, rim[5]+2, x2, y2);
		alpha += da;
		line(board, r, r, (int)x2, (int)y2, yellow);
	}

	circlefill(board, r, r, (int)FRAC(r, rim[1]), yellow);
	circlefill(board, r, r, (int)FRAC(r, rim[1]-1), red);
	circlefill(board, r, r, (int)FRAC(r, rim[0]), black);
	circle(board, r, r, (int)FRAC(r, rim[2]), yellow);
	circle(board, r, r, (int)FRAC(r, rim[3]), yellow);
	circle(board, r, r, (int)FRAC(r, rim[4]), yellow);
	circle(board, r, r, (int)FRAC(r, rim[5]), yellow);

	int numbers[] = { 6,10,15,2,17,3,19,7,16,8,11,14,9,12,5,20,1,18,4,13 };
	da = 18;
	alpha = 0;
	for (int i=0; i<20; ++i) {
		GetPoint(alpha, r, rim[5]+3, x2, y2);
		y2 -= text_height(font)/2.0f;
		alpha += da;
		textprintf_centre_ex(board, font, (int)x2, (int)y2, white, -1, "%d", numbers[i]);

		GetPoint(alpha, r, (rim[1]+rim[2])>>1, x2, y2);
		floodfill(board, (int)x2, (int)y2, i%2 ? black: red);
		GetPoint(alpha, r, (rim[2]+rim[3])>>1, x2, y2);
		floodfill(board, (int)x2, (int)y2, i%2 ? red: black);
		GetPoint(alpha, r, (rim[3]+rim[4])>>1, x2, y2);
		floodfill(board, (int)x2, (int)y2, i%2 ? black: red);
		GetPoint(alpha, r, (rim[4]+rim[5])>>1, x2, y2);
		floodfill(board, (int)x2, (int)y2, i%2 ? red :black);
	}
}


int Board::GetPoints(int mx, int my, int &f) {
	int r = (board->w)>>1;

	int d = (int)sqrt((mx - r)*(mx - r) + (my - r)*(my - r));
	d *= 100;
	d /= r;

	f = 1;
	int smp = SMP_HIT;
	Sound::Play(SMP_HIT, 1000 + (rand()%50-25));
	if (d < rim[0]) {
		Sound::Play(SMP_50);
		MarkHit(mx, my);
		return 50;
	}
	else if (d < rim[1]) {
		Sound::Play(SMP_25);
		MarkHit(mx, my);
		return 25;
	}
	else if (d >= rim[2] && d < rim[3]) {
		smp = SMP_TRIPLE;
		f = 3;
	}
	else if (d >= rim[4] && d < rim[5]) {
		smp = SMP_DOUBLE;
		f = 2;
	}
	else if (d >= rim[5]) {
		Sound::Play(SMP_HIT, 1000 + (rand()%50-25));
		MarkHit(mx, my);
		return 0;
	}

	if (smp != SMP_HIT) {
		Sound::Play(smp);
	}
	MarkHit(mx, my);
	int numbers[] = { 6,10,15,2,17,3,19,7,16,8,11,14,9,12,5,20,1,18,4,13,6 };
	mx -= r;
	my -= r;
	if (mx == 0) mx = 1;
	float fmx = mx == 0 ? 0.0001f : (float)mx;
	float fmy = my == 0 ? 0.0001f : (float)my;
	float a = DEG(atan(ABS(fmy/fmx)));
	if (fmx < 0.0f && fmy > 0.0f) {
		a = 180.0f - a;
	}
	else if (fmx < 0.0f && fmy < 0.0f) {
		a += 180.0f;
	}
	else if (fmx > 0.0f && fmy < 0.0f) {
		a = 360.0f - a;
	}
	a += 9.0f;
	a /= 18.0f;
	return numbers[(int)a]*f;
}


void Board::MarkHit(int mx, int my) {
	draw_sprite(board, mark, mx-mark->w/2, my-mark->h/2);
}



class Dart : public Object {
	protected:
		int size;
		int defx, defy;
		BITMAP *sprite;
		int angle;

	public:
		Dart();
		~Dart();

		void Draw(BITMAP *buffer);
		void Move(int x, int y);
		void InitDefaultPosition(int x, int y);
		void Reset();
};


Dart::Dart() : Object(0.0f, 0.0f), size(8), angle(0) {
	sprite = (BITMAP *)darts_data[BMP_DART].dat;
}


Dart::~Dart() {}


void Dart::Draw(BITMAP *buffer) {
	//rotate_sprite(buffer, sprite, (int)x - size, (int)y - size, itofix(angle));
	pivot_sprite(buffer, sprite, (int)x, (int)y, sprite->w>>1, sprite->h>>1, itofix(angle));
}


void Dart::Move(int xx, int yy) {
	angle += abs((int)(yy - y));
	angle %= 256;
	x = xx;
	y = yy;
}


void Dart::InitDefaultPosition(int xx, int yy) {
	defx = xx;
	defy = yy;
}


void Dart::Reset() {
	Move(defx, defy);
}


class ScoreBoard : public Object {
	protected:
		BITMAP *sc1, *sc2;
		BITMAP *font;

		void DrawNumber(BITMAP *buffe, int num, int x, int y);

	public:
		ScoreBoard(int x, int y);
		~ScoreBoard();

		void Draw(BITMAP *buffer, int activePlayer, int p1s, int p2s, int lastScore, int total);
		bool CanDraw() const { return (sc1 && sc2 && font); }
};


ScoreBoard::ScoreBoard(int x, int y) : Object(x, y) {
	sc1 = (BITMAP *)darts_data[BMP_SCORE1].dat;
	sc2 = (BITMAP *)darts_data[BMP_SCORE2].dat;
	font = (BITMAP *)darts_data[BMP_NUMBERS].dat;
	/*
	sc1 = load_bitmap("score1.tga", NULL);
	sc2 = load_bitmap("score2.tga", NULL);
	font = load_bitmap("numbers.tga", NULL);
	*/
}


ScoreBoard::~ScoreBoard() {
	/*
	destroy_bitmap(sc1);
	destroy_bitmap(sc2);
	destroy_bitmap(font);
	*/
}


void ScoreBoard::Draw(BITMAP *buffer, int activePlayer, int p1s, int p2s, int lastScore, int total) {
	int x = (int)(this->x);
	int y = (int)(this->y);
	int w = sc1->w/2;
	int h = sc1->h/4;

	// the active player display
	blit(activePlayer == 0 ? sc2 : sc1, buffer, 0, 0, x, y, w, h);
	blit(activePlayer == 1 ? sc2 : sc1, buffer, w, 0, x+w, y, w, h);

	// the scores
	blit(sc1, buffer, 0, h, x, y+h, 2*w, h);
	DrawNumber(buffer, p1s, x+6, y+h+4);
	DrawNumber(buffer, lastScore, x+6+w/2, y+h+4);
	DrawNumber(buffer, total, x+6+w, y+h+4);
	DrawNumber(buffer, p2s, x+6+3*w/2, y+h+4);

	// the selected game
	int game = 0;
	switch (Params::maxPoints) {
		case 301:	game = 0;	break;
		case 501:	game = 1;	break;
		case 701:	game = 2;	break;
		case 901:	game = 3;	break;
	}
	for (int i=0; i<4; ++i) {
		blit(i==game ? sc2 : sc1, buffer, i*w/2, 2*h, x+i*w/2, y+2*h, w/2, h);
	}

	// game type
	blit(Params::doubleIn == 1 ? sc2 : sc1, buffer, 0, 3*h, x, y+3*h, w, h);
	blit(Params::doubleOut == 1 ? sc2 : sc1, buffer, w, 3*h, x+w, y+3*h, w, h);
}


void ScoreBoard::DrawNumber(BITMAP *buffer, int n, int x, int y) {
	int w = font->w/10;
	int h = font->h;
	int d;

	d = n/100;
	masked_blit(font, buffer, d*w, 0, x, y, w, h);

	d = (n/10)%10;
	masked_blit(font, buffer, d*w, 0, x+w+1, y, w, h);

	d = n%10;
	masked_blit(font, buffer, d*w, 0, x+2*(w+1), y, w, h);
}



class DartGame : public IGame {
	private:
		BITMAP *holder;
		Board *board;
		Pad *pad;
		ScoreBoard *scoreBoard;
		Player *player[2];
		Dart *dart[3];
		int lastScore, total;
		int throwing;
		int activePlayer;
		int counter;
		int mouse_x, mouse_y;
		bool paused;
		bool game_over;

		void Draw();
		bool Update();
		void DrawDoubleBuffer();
		void ResetDarts();
		void GameOver();
		void RestartGame();

	public:
		DartGame();
		~DartGame();

		char *GetName() {
			return "Allegro Darts";
		}

		char *GetAuthor() {
			return "Miran Amon";
		}

		char *GetDescription() {
			return "Use the mouse to grab the darts in the holder in the bottom right of the screen and drag them to the pad above. The pad represents the target on the left so you can't really see where you're throwing. You need to practice to get the feel for it :)";
		}

		char *GetVersion() {
			return "1.02";
		}

		char *GetIconPath() {
			return "darts.bmp";
		}

		bool Init() {
			if (!IGame::Init()) {
				return false;
			}

			// custom initialization goes here
			switch (darts_settings[0].value) {
				case 0:		Params::maxPoints = 301;		break;
				case 1:		Params::maxPoints = 501;		break;
				case 2:		Params::maxPoints = 701;		break;
				case 3:		Params::maxPoints = 901;		break;
			};
			Params::doubleIn = darts_settings[1].value;
			Params::doubleOut = darts_settings[2].value;

			darts_data = load_datafile("games/darts/darts.dat");
			if (!darts_data) {
				return false;
			}

			board = new Board(Params::h);
			pad = new Pad(Params::h, (5*Params::h-3*Params::w)/2, Params::w-Params::h);
			for (int i=0; i<3; i++) {
				dart[i] = new Dart();
			}
			int dx = (Params::w-Params::h)/3;
			dart[0]->InitDefaultPosition(Params::h + dx/2, Params::h - (Params::w-Params::h)/4);
			dart[1]->InitDefaultPosition(Params::h + 3*dx/2, Params::h - (Params::w-Params::h)/4);
			dart[2]->InitDefaultPosition(Params::h + 5*dx/2, Params::h - (Params::w-Params::h)/4);
			ResetDarts();

			player[0] = new Player;
			player[1] = new Player;
			scoreBoard = new ScoreBoard(Params::h, 0);
			holder = (BITMAP *)darts_data[BMP_HOLDER].dat;
			//holder = load_bitmap("holder.tga", NULL);
			Sound::Initialize();
			player[activePlayer]->StartTurn();
			paused = false;
			game_over = false;
			return true;
		}

		void Deinit() {
			// custom deinitialization goes here
			if (board) delete board;
			if (pad) delete pad;
			if (scoreBoard) delete scoreBoard;
			for (int i=0; i<3; i++) {
				if (dart[i]) {
					delete dart[i];
					dart[i] = 0;
				}
			}
			if (player[0]) delete player[0];
			if (player[1]) delete player[1];

			unload_datafile(darts_data);
			darts_data = 0;

			// no more Allegro calls after this line!
			IGame::Deinit();
		}

		void Draw(BITMAP *canvas);
		int Logic(int keys[8], int mouse[4], float dT);

		int Score() {
			// Return -1 if this game doesn't have points, otherwise the last
			// valid score where more points is better.
			return -1;
		}

		bool UseSystemMouse() {
			// Return false if the game draws its own cursor or doesn't use
			// the mouse at all.
			return true;
		}

		void Pause() {
			paused = true;
		}

		void Resume() {
			paused = false;
		}

		GameSettings* GetSettings() {
			return darts_settings;
		}
};


DartGame::DartGame()
	:IGame(),
	lastScore(0),
	total(0),
	throwing(0),
	activePlayer(0),
	counter(-1)
{
}


DartGame::~DartGame() {
}


int DartGame::Logic(int keys[8], int mouse[4], float dT) {
	mouse_x = mouse[0] - (SCREEN_W - Params::w)/2;
	mouse_y = mouse[1] - (SCREEN_H - Params::h)/2;

	if (!paused && !game_over) {
		int mouse_b = mouse[2];

		if (mouse_b && pad->CanThrow(mouse_x, mouse_y) && counter==-1) {
			Params::fps = (int)(1000.0f / dT);
			throwing = Params::fps/(Params::difficulty+4);
		}

		if (throwing) {
			int x, y;
			--throwing;
			if (pad->GetMousePos(x, y, mouse_x, mouse_y, mouse_b) || !throwing) {
				throwing = 0;
				int f;
				lastScore = board->GetPoints(x,y,f);
				player[activePlayer]->Throw();
				player[activePlayer]->AwardPoints(lastScore,f);
				if (player[activePlayer]->Score() != Params::maxPoints) {
					total += lastScore;
				}
				if (player[activePlayer]->Score() == 0) {
					Sound::Play(SMP_WIN);
					GameOver();
					//return 1;
					game_over = true;
					return 0;
				}
				if (!player[activePlayer]->DartsLeft()) {
					counter = 2*Params::fps;
				}
			}
		}
	}

	if (counter > 0) {
		--counter;
	}
	else if (counter == 0) {
		ResetDarts();
		board->DrawBoard();
		counter = -1;
		lastScore = 0;
		total = 0;
		++activePlayer;
		activePlayer %= 2;
		player[activePlayer]->StartTurn();
	}

	if (game_over) {
		if (key[4] || keys[5] || keys[6] || keys[7]) {
			int tmp = activePlayer;
			++tmp;
			tmp%=2;
			ResetDarts();
			RestartGame();
			activePlayer = tmp;
			player[activePlayer]->StartTurn();
			game_over = false;
		}
	}

	return 0;
}


void DartGame::Draw(BITMAP *canvas) {
	int offx = (SCREEN_W - Params::w)/2;
	int offy = (SCREEN_H - Params::h)/2;
	BITMAP *buffer = create_sub_bitmap(canvas, offx, offy, Params::w, Params::h);

	// clear the buffer
	clear_to_color(canvas, makecol(160,160,160));

	// do game specific drawing
	board->Draw(buffer);
	pad->Draw(buffer);
	scoreBoard->Draw(buffer, activePlayer, player[0]->Score(), player[1]->Score(), lastScore, total);
	if (holder) blit(holder, buffer, 0, 0, Params::h, Params::h - (Params::w-Params::h)/2, holder->w, holder->h);
	for (int i=0; i<3; ++i) {
		dart[i]->Draw(buffer);
	}

	if (game_over) {
		rectfill(canvas, SCREEN_W/4, SCREEN_H/2-4, 3*SCREEN_W/4, SCREEN_H/2+28, makecol(128,128,128));
		rect(canvas, SCREEN_W/4, SCREEN_H/2-4, 3*SCREEN_W/4, SCREEN_H/2+28, makecol(192,192,192));
		textprintf_centre_ex(canvas, font, SCREEN_W/2, SCREEN_H/2, makecol(192,192,192), -1, "Game over! Player %d won!", activePlayer+1);
		textprintf_centre_ex(canvas, font, SCREEN_W/2, SCREEN_H/2+16, makecol(192,192,192), -1, "press any key to continue...");
	}

	// draw the mouse
	if (throwing) {
		dart[player[activePlayer]->DartsLeft()-1]->Move(mouse_x, mouse_y);
	}
	else {
		/*
		if (!menuOn) {
			draw_sprite(buffer, Data::cursor, mouse_x, mouse_y);
		}
		*/
	}

	destroy_bitmap(buffer);
}



void DartGame::ResetDarts() {
	for (int i=0; i<3; ++i) {
		dart[i]->Reset();
	}
}


void DartGame::GameOver() {
	//Draw();
	//RestartGame();
}


void DartGame::RestartGame() {
	player[0]->Reset();
	player[1]->Reset();
	ResetDarts();
	board->DrawBoard();
	activePlayer = 0;
	lastScore = 0;
	total = 0;
	throwing = 0;
	counter = -1;
	player[activePlayer]->StartTurn();
}


extern "C" DLLEXPORT IGame* GetPlugin() {
	return new DartGame;
}
